/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.i2s.impl;

import java.util.Iterator;
import java.util.Map;
import java.util.HashMap;

import org.xmloperator.framework.document.relaxng.Name;
import org.xmloperator.framework.document.relaxng.Pattern;
import org.xmloperator.framework.document.relaxng.NonEmptyPattern;
import org.xmloperator.i2s.framework.PatternCategory;
import org.xmloperator.i2s.framework.AttributesPatternCategory;
import org.xmloperator.i2s.framework.RepeatableName;
import org.xmloperator.i2s.util.PatternFactory;

/**
 * A pattern category for attribute patterns, disregarding elements.
 *
 * Formally :
 * attribute ::= <attribute name="QName"> <text/> </attribute>
 * attributes ::= <empty/> | attribute | (<group> attribute+ </group>)
 */
public final class AttributesPatternCategoryImpl
    implements AttributesPatternCategory {
  private int patternCount = 0;
  private final Map fromAttributeNameToAttributeUsage = new HashMap();

  private static final class AttributeUsage {
    public int usageCount = 0;
  }

  public AttributesPatternCategoryImpl() {
  }

  /////////////////////////////////
  // PatternCategory implementation
  /////////////////////////////////

  public final Pattern fullPattern(Map fromNameToElementDefinition) {
    return this.fullPattern();
  }

  public final boolean addPattern(
      Name[] attributeNames, boolean hasText,
      RepeatableName[] repeatableElementNames) {
    this.addAttributeNames(attributeNames);
    return true;
  }

  ///////////////////////////////////////////
  // AttributesPatternCategory implementation
  ///////////////////////////////////////////

  public void addAttributeNames(Name[] attributeNames) {
    this.patternCount++;
    for (int k = 0; k < attributeNames.length; k++) {
      Name attributeName = attributeNames[k];
      AttributeUsage attributeUsage =
          (AttributeUsage)this.fromAttributeNameToAttributeUsage.
          get(attributeName);
      if (attributeUsage == null) {
        attributeUsage = new AttributeUsage();
        this.fromAttributeNameToAttributeUsage.put(
            attributeName, attributeUsage);
      }
      attributeUsage.usageCount++;
    }
  }

  public final Pattern fullPattern() {
    int attributeNameCount = this.fromAttributeNameToAttributeUsage.size();
    if (attributeNameCount == 0)
      return PatternFactory.emptyPattern();
    NonEmptyPattern[] childPatterns = new NonEmptyPattern[attributeNameCount];
    int index = 0;
    Iterator iterator =
        this.fromAttributeNameToAttributeUsage.keySet().iterator();
    while (iterator.hasNext()) {
      Name attributeName = (Name)iterator.next();
      AttributeUsage attributeUsage =
          (AttributeUsage)this.fromAttributeNameToAttributeUsage.
          get(attributeName);
      boolean isOptional = attributeUsage.usageCount < this.patternCount;
      childPatterns[index++] =
          PatternFactory.namedAttribute(attributeName, isOptional);
    }
    return attributeNameCount == 1 ? childPatterns[0] :
                                     PatternFactory.group(childPatterns);
  }
}
